@@ -69,19 +69,6 @@ mapi_backend_get_settings (EMapiBackend *backend)
 	return CAMEL_MAPI_SETTINGS (settings);
 }
 
-static void
-mapi_backend_queue_auth_session (EMapiBackend *backend)
-{
-	backend->priv->need_update_folders = FALSE;
-
-	/* For now at least, we don't need to know the
-	 * results, so no callback function is needed. */
-	e_backend_authenticate (
-		E_BACKEND (backend),
-		E_SOURCE_AUTHENTICATOR (backend),
-		NULL, NULL, NULL);
-}
-
 struct SyndFoldersData
 {
 	EMapiBackend *backend;
@@ -112,7 +99,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 	ESourceRegistryServer *server;
 	EMapiBackend *backend;
 	GSList *mapi_folders;
-	gboolean has_gal = FALSE;
+	gboolean has_gal = FALSE, is_online;
 	gint color_seed;
 
 	g_return_val_if_fail (sfd != NULL, FALSE);
@@ -121,6 +108,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 
 	backend = sfd->backend;
 	mapi_folders = sfd->folders;
+	is_online = e_backend_get_online (E_BACKEND (backend));
 
 	server = e_collection_backend_ref_server (E_COLLECTION_BACKEND (backend));
 	all_sources = e_source_registry_server_list_sources (server, NULL);
@@ -192,7 +180,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 	}
 
 	/* those which left are either mail sources, GAL or removed from the server */
-	for (citer = configured; citer; citer = citer->next) {
+	for (citer = configured; citer && is_online; citer = citer->next) {
 		ESource *source = citer->data;
 		ESourceMapiFolder *folder_ext;
 		const gchar *foreign_user_name;
@@ -232,6 +220,7 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 		ESource *source = citer->data;
 		ESourceMapiFolder *extension;
 		const gchar *foreign_username;
+		gboolean remove = FALSE;
 
 		if (!e_source_has_extension (source, E_SOURCE_EXTENSION_MAPI_FOLDER))
 			continue;
@@ -251,9 +240,23 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 			if (has_gal)
 				e_source_registry_server_add_source (server, source);
 			else
-				e_source_remove_sync (source, NULL, NULL);
+				remove = TRUE;
 		} else {
-			e_source_remove_sync (source, NULL, NULL);
+			remove = TRUE;
+		}
+
+		if (remove) {
+			/* in online mode remove means remove, while in offline mode
+			   there are used only discovered sources from the last run,
+			   thus re-add them all
+			*/
+			if (is_online) {
+				e_source_remove_sync (source, NULL, NULL);
+			} else {
+				e_server_side_source_set_writable (E_SERVER_SIDE_SOURCE (source), TRUE);
+				e_server_side_source_set_remote_deletable (E_SERVER_SIDE_SOURCE (source), TRUE);
+				e_source_registry_server_add_source (server, source);
+			}
 		}
 	}
 	g_list_free_full (all_sources, g_object_unref);
@@ -300,6 +303,71 @@ mapi_backend_sync_folders_idle_cb (gpointer user_data)
 	return FALSE;
 }
 
+static ESourceAuthenticationResult
+mapi_backend_try_password_sync (ESourceAuthenticator *authenticator,
+				const GString *password,
+				GCancellable *cancellable,
+				GError **error);
+
+static gpointer
+mapi_backend_authenticate_kerberos_thread (gpointer user_data)
+{
+	EMapiBackend *mapi_backend = user_data;
+	CamelMapiSettings *mapi_settings;
+
+	g_return_val_if_fail (E_IS_MAPI_BACKEND (mapi_backend), NULL);
+
+	mapi_settings = mapi_backend_get_settings (mapi_backend);
+	e_mapi_util_trigger_krb_auth_from_settings (mapi_settings, NULL);
+
+	mapi_backend_try_password_sync (E_SOURCE_AUTHENTICATOR (mapi_backend), NULL, NULL, NULL);
+
+	g_object_unref (mapi_backend);
+
+	return NULL;
+}
+
+static void
+mapi_backend_queue_auth_session (EMapiBackend *backend)
+{
+	CamelMapiSettings *mapi_settings;
+
+	mapi_settings = mapi_backend_get_settings (backend);
+
+	if (!e_backend_get_online (E_BACKEND (backend))) {
+		struct SyndFoldersData *sfd;
+
+		sfd = g_new0 (struct SyndFoldersData, 1);
+		sfd->folders = NULL;
+		sfd->backend = g_object_ref (backend);
+		sfd->profile = camel_mapi_settings_dup_profile (mapi_settings);
+
+		mapi_backend_sync_folders_idle_cb (sfd);
+		sync_folders_data_free (sfd);
+
+		return;
+	}
+
+	backend->priv->need_update_folders = FALSE;
+
+	/* kerberos doesn't use passwords, do it directly */
+	if (camel_mapi_settings_get_kerberos (mapi_settings)) {
+		GThread *thread;
+
+		thread = g_thread_new (NULL, mapi_backend_authenticate_kerberos_thread, g_object_ref (backend));
+		g_thread_unref (thread);
+
+		return;
+	}
+
+	/* For now at least, we don't need to know the
+	 * results, so no callback function is needed. */
+	e_backend_authenticate (
+		E_BACKEND (backend),
+		E_SOURCE_AUTHENTICATOR (backend),
+		NULL, NULL, NULL);
+}
+
 static void
 mapi_backend_source_changed_cb (ESource *source,
 				EMapiBackend *backend)
@@ -375,8 +443,7 @@ mapi_backend_populate (ECollectionBackend *backend)
 	mapi_backend->priv->need_update_folders = TRUE;
 
 	/* do not do anything, if account is disabled */
-	if (!e_source_get_enabled (source) ||
-	    !e_backend_get_online (E_BACKEND (backend)))
+	if (!e_source_get_enabled (source))
 		return;
 
 	/* We test authentication passwords by attempting to synchronize
@@ -430,13 +497,6 @@ mapi_backend_child_added (ECollectionBackend *backend,
 	extension_name = E_SOURCE_EXTENSION_MAIL_TRANSPORT;
 	is_mail |= e_source_has_extension (child_source, extension_name);
 
-	/* Synchronize mail-related display names with the collection. */
-	if (is_mail)
-		g_object_bind_property (
-			collection_source, "display-name",
-			child_source, "display-name",
-			G_BINDING_SYNC_CREATE);
-
 	/* Synchronize mail-related user with the collection identity. */
 	extension_name = E_SOURCE_EXTENSION_AUTHENTICATION;
 	if (is_mail && e_source_has_extension (child_source, extension_name)) {
@@ -808,6 +868,10 @@ e_mapi_backend_init (EMapiBackend *backend)
 		g_str_equal,
 		g_free,
 		g_object_unref);
+
+	g_signal_connect (
+		backend, "notify::online",
+		G_CALLBACK (mapi_backend_populate), NULL);
 }
 
 void
